/*
 * This program is free software; you can redistribute it
 * and/or modify it under the terms of the Shared Code
 * License (SCL) as published by www.IntellectualHeaven.com;
 * either version 1.0, or (at your option) any later version.
 *
 * You should have received a copy of the SCL 1.0 license,
 * look for file license.txt, along with this software; if not,
 * then please write to xx_pankaj_xx@intellectualheaven.com
 * for an electronic copy of the same.
 */

/* ++
History (in reverse order)

xx_pankaj_xx@intellectualheaven.com   2003-08-18
First version created and checked in CVS
-- */

// L_GENERIC_SOCKET.cpp: implementation of the CL_GENERIC_SOCKET class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "L_GENERIC_SOCKET.h"
#include "all_defines.h"

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CL_GENERIC_SOCKET::CL_GENERIC_SOCKET()
{

}

CL_GENERIC_SOCKET::~CL_GENERIC_SOCKET()
{

}

/*!
 * This function provides facility to receive a fixed number of bytes
 * from a socket descriptor. It also provides timeout feature and wait
 * for 2 times the arg_timeout.
 */
int CL_GENERIC_SOCKET::Receive(		SOCKET arg_sock,
									void *arg_data,
									int arg_len,
									int arg_timeout,
									int n_tries,
									int flag
							   )
{
	char *data = (char *)arg_data;
	int n_tried_number = 0;

	fd_set my_fd_set;
	FD_ZERO( &my_fd_set);
	FD_SET(	arg_sock,
			&my_fd_set);

	timeval time_out;
	time_out.tv_sec		= arg_timeout;
	time_out.tv_usec	= 0;

	if(	arg_timeout == 0)
		n_tries = 0;

	int n_bytes_recvd = 0;
	int n_total_bytes_recvd = 0;
	int n_res = 0;

	while(	arg_len > 0 
			&& (	n_tries == 0 
					|| n_tried_number < n_tries
				)
			)
	{
		if(	arg_timeout == 0 )
		{
			n_res	= select(	255,
								&my_fd_set,
								NULL,
								NULL,
								NULL);
		}
		else
		{
			n_res	= select(	255,
								&my_fd_set,
								NULL,
								NULL,
								&time_out);
			n_tried_number++;
		}

		if( n_res == SOCKET_ERROR)
		{
			return n_total_bytes_recvd;
		}
		else
		{
			n_bytes_recvd = recv(	arg_sock,
									&data[n_total_bytes_recvd],
									arg_len,
									flag);

			if( n_bytes_recvd == SOCKET_ERROR ||
				n_bytes_recvd == 0)
			{
				return -1;
			}
			else
			{
				arg_len				-= n_bytes_recvd;
				n_total_bytes_recvd	+= n_bytes_recvd;
			}
		}
	}
	return n_total_bytes_recvd;
}


int CL_GENERIC_SOCKET::Send(	SOCKET arg_sock,
								void *arg_data,
								int arg_len,
								int flag
							   )
{
	int n_bytes_sent		= 0;
	int n_total_bytes_sent	= 0;
	const char *data = (const char *)arg_data;

	char sz_size[SIZE_INT];
	memcpy(	sz_size,
			&arg_len,
			SIZE_INT);

	n_bytes_sent = send(	arg_sock,
							sz_size,
							SIZE_INT,
							flag);

	if(n_bytes_sent < SIZE_INT)
	{
		return -1;
	}

	n_bytes_sent = 0;
	while(arg_len > 0)
	{
		n_bytes_sent = send(	arg_sock,
								&data[n_total_bytes_sent],
								arg_len,
								flag);

		if(n_bytes_sent == SOCKET_ERROR)
		{
			return n_total_bytes_sent;
		}
		else
		{
			arg_len				-= n_bytes_sent;
			n_total_bytes_sent	+= n_bytes_sent;
		}
	}
	return n_total_bytes_sent;
}
